use rocket::{get, Responder, State};
use rocket::serde::json::Json;
use serde::Serialize;

use crate::ctx;
use crate::ctx::Ctx;

#[derive(Serialize)]
pub struct WebFinger {
    subject: String,
    aliases: Vec<String>,
}

#[derive(Responder)]
pub enum WebFingerResponse {
  #[response(status = 200, content_type = "json")]
  Found(Json<WebFinger>),

  #[response(status = 404)]
  Missing(String),

  #[response(status = 400)]
  ParseError(String),

  #[response(status = 500)]
  InternalError(String),
}

#[get("/?<resource>")]
pub fn main(ctx: &State<Ctx>, resource: &str) -> WebFingerResponse {
  match parse_resource(&resource) {
    Ok(("acct", idstr)) | Ok(("", idstr)) => match ctx::user::parse_id(idstr) {
      Ok(id) => match ctx.db.exists(&id) {
        Ok(true) => WebFingerResponse::Found(Json(WebFinger {
          subject: resource.to_owned(),
          aliases: vec![resource.to_owned()],
        })),

        Ok(false) => WebFingerResponse::Missing(format!("unknown resource: {}", resource)),
        Err(msg) => WebFingerResponse::InternalError(format!("error: {}", msg))
      }
      Err(msg) => WebFingerResponse::ParseError(format!("error: {}", msg)),
    },
    Ok((kind, _)) => WebFingerResponse::ParseError(format!("error: unknown kind, {}", kind)),
    Err(msg) => WebFingerResponse::ParseError(format!("error: {}", msg)),
  }
}

fn parse_resource(v: &str) -> Result<(&str, &str), String> {
  let mut toks = v.split(':');
  match (toks.next(), toks.next(), toks.next()) {
    (Some(id),   None,     None) => Ok(("",   id)),
    (Some(kind), Some(id), None) => Ok((kind, id)),
    _ => Err(format!("invalid resource specifier: {}", v)),
  }
}
